# Program to test network and produce log files with timestamps and number of packets received/sent.
# Created by Gee for testing the game Little Brats!
#
# https://studios.ptilouk.net

extends Node

var started: bool = false
var frame_id: int = 0
var current_idx: int = 0
var latest_time: int = 0
var file: FileAccess
var data: Array = []
var times: Array = []
var nb_lost: int = 0
var nb_received: int = 0

# To run these tests, run two instances from the Godot engine, one of them with --server as an argument

func _ready() -> void:
    data.resize(1024)
    print(OS.get_user_data_dir())
    if '--server' in OS.get_cmdline_args():
        var peer: MultiplayerPeer = ENetMultiplayerPeer.new()
        peer.create_server(27000)
        print('Server created')
        multiplayer.set_multiplayer_peer(peer)
        file = FileAccess.open('user://server_stats.log', FileAccess.WRITE)
    else:
        var peer: MultiplayerPeer = ENetMultiplayerPeer.new()
        peer.create_client('localhost', 27000)
        multiplayer.set_multiplayer_peer(peer)
        print('Client created')
        await multiplayer.connected_to_server
        print('Client connected')
        file = FileAccess.open('user://clients_stats.log', FileAccess.WRITE)
        start.rpc()
    multiplayer.server_disconnected.connect(stop)


func _physics_process(delta: float) -> void:
    if not started:
        return

    if '--server' in OS.get_cmdline_args():
        if current_idx > 50:
            stop.rpc()
            await stop()

        frame_id += 1
        if frame_id % 4 == 0:
            var time: float = Time.get_unix_time_from_system()
            for idx: int in range(data.size()):
                data[idx] = randi_range(0, 10000)
            send_data.rpc(current_idx,data)
            file.store_string(str(time) + ' -1\n')
            latest_time = time
            current_idx += 1
    else:
        if nb_received != 0:
            var time: float = Time.get_unix_time_from_system()
            file.store_string(str(time) + ' ' + str(nb_received) + '\n')
            nb_received = 0


@rpc('any_peer', 'call_local', 'reliable')
func start() -> void:
    started = true


@rpc('authority', 'call_remote', 'reliable')
func stop() -> void:
    file.flush()
    await get_tree().create_timer(1).timeout
    get_tree().quit()


@rpc("authority", "call_remote", "reliable") # Change to unreliable to test unreliable
func send_data(idx: int, ndata: Array) -> void:
    for i: int in range(data.size()):
        data[i] = ndata[i]
    nb_received += 1
